/*
 *
 * @Component			OMAPCONF
 * @Filename			clkdm_dependency54xx-data.c
 * @Description			OMAP5 ES2 Clock Domain Dependencies Definitions
 * @Author			Patrick Titiano (p-titiano@ti.com)
 * @Date			2012
 * @Copyright			Texas Instruments Incorporated
 *
 *
 * Copyright (C) 2012 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <clkdm_dependency54xx-data.h>
#include <cm54xx-defs.h>

/*
 * Table used to retrieve clock domain dependency register.
 * NULL = does not exist
 */
const reg *clkdmdep54xx_reg_table[CLKDM54XX_ID_MAX][2] = {
	{NULL, &omap5430_cm_emu_dynamicdep}, /* CLKDM54XX_EMU */
	{NULL, NULL}, /* CLKDM54XX_WKUPAON */
	{NULL, NULL}, /* CLKDM54XX_COREAON */
	{&omap5430_cm_cam_staticdep, &omap5430_cm_cam_dynamicdep}, /* CLKDM54XX_CAM */
	{NULL, &omap5430_cm_l4cfg_dynamicdep}, /* CLKDM54XX_L4CFG */
	{NULL, NULL}, /* CLKDM54XX_EMIF */
	{&omap5430_cm_ipu_staticdep, &omap5430_cm_ipu_dynamicdep}, /* CLKDM54XX_IPU */
	{NULL, &omap5430_cm_l3main2_dynamicdep}, /* CLKDM54XX_L3MAIN2 */
	{NULL, NULL}, /* CLKDM54XX_INSTR */
	{NULL, &omap5430_cm_l3main1_dynamicdep}, /* CLKDM54XX_L3MAIN1 */
	{&omap5430_cm_c2c_staticdep, &omap5430_cm_c2c_dynamicdep}, /* CLKDM54XX_C2C */
	{&omap5430_cm_dma_staticdep, &omap5430_cm_dma_dynamicdep}, /* CLKDM54XX_DMA */
	{&omap5430_cm_mipiext_staticdep, &omap5430_cm_mipiext_dynamicdep}, /* CLKDM54XX_MIPIEXT */
	{&omap5430_cm_dss_staticdep, &omap5430_cm_dss_dynamicdep}, /* CLKDM54XX_DSS */
	{NULL, NULL}, /* CLKDM54XX_CUST_EFUSE */
	{&omap5430_cm_l3init_staticdep, &omap5430_cm_l3init_dynamicdep}, /* CLKDM54XX_L3INIT */
	{NULL, &omap5430_cm_l4per_dynamicdep}, /* CLKDM54XX_L4PER */
	{&omap5430_cm_l4sec_staticdep, &omap5430_cm_l4sec_dynamicdep}, /* CLKDM54XX_L4SEC */
	{NULL, NULL}, /* CLKDM54XX_ABE */
	{&omap5430_cm_dsp_staticdep, &omap5430_cm_dsp_dynamicdep}, /* CLKDM54XX_DSP */
	{&omap5430_cm_gpu_staticdep, &omap5430_cm_gpu_dynamicdep}, /* CLKDM54XX_GPU */
	{&omap5430_cm_iva_staticdep, &omap5430_cm_iva_dynamicdep}, /* CLKDM54XX_IVA */
	{&omap5430_cm_mpu_staticdep, &omap5430_cm_mpu_dynamicdep}, /* CLKDM54XX_MPU */
	{NULL, NULL} }; /* CLKDM54XX_NONE */


/*
 * Table used to retrieve dependency bit position in dependency register
 * -1 = does not exist
 */
const short int clkdmdep54xx_bit_pos_table[CLKDM54XX_ID_MAX] = {
	-1, /* CLKDM54XX_EMU */
	15, /* CLKDM54XX_WKUPAON */
	16, /* CLKDM54XX_COREAON */
	9, /* CLKDM54XX_CAM */
	12, /* CLKDM54XX_L4CFG */
	4, /* CLKDM54XX_EMIF */
	0, /* CLKDM54XX_IPU */
	6, /* CLKDM54XX_L3MAIN2 */
	-1, /* CLKDM54XX_INSTR */
	5, /* CLKDM54XX_L3MAIN1 */
	18, /* CLKDM54XX_C2C */
	-1, /* CLKDM54XX_DMA */
	21, /* CLKDM54XX_MIPIEXT */
	8, /* CLKDM54XX_DSS */
	-1, /* CLKDM54XX_CUST_EFUSE */
	7, /* CLKDM54XX_L3INIT */
	13, /* CLKDM54XX_L4PER */
	14, /* CLKDM54XX_L4SEC */
	3, /* CLKDM54XX_ABE */
	1, /* CLKDM54XX_DSP */
	10, /* CLKDM54XX_GPU */
	2, /* CLKDM54XX_IVA */
	-1, /* CLKDM54XX_MPU */
	-1}; /* CLKDM54XX_NONE */


const clkdmdep_ctrl_type clkdmdep54xx_emu_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_wkupaon_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_coreaon_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_cam_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RW, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_RW, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_l4cfg_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_emif_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_ipu_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_RW, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_RO, /* CLKDM54XX_COREAON */
	CLKDMDEP_RO, /* CLKDM54XX_CAM */
	CLKDMDEP_RW, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RW, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_RW, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_RW, /* CLKDM54XX_L3INIT */
	CLKDMDEP_RW, /* CLKDM54XX_L4PER */
	CLKDMDEP_RW, /* CLKDM54XX_L4SEC */
	CLKDMDEP_RW, /* CLKDM54XX_ABE */
	CLKDMDEP_RW, /* CLKDM54XX_DSP */
	CLKDMDEP_RW, /* CLKDM54XX_GPU */
	CLKDMDEP_RW, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_l3main2_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_instr_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_l3main1_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_c2c_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_RW, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RW, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_RW, /* CLKDM54XX_L3INIT */
	CLKDMDEP_RW, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_RW, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_RW, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_dma_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_RW, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_RO, /* CLKDM54XX_CAM */
	CLKDMDEP_RW, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RW, /* CLKDM54XX_EMIF */
	CLKDMDEP_RW, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_RW, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_RW, /* CLKDM54XX_L3INIT */
	CLKDMDEP_RW, /* CLKDM54XX_L4PER */
	CLKDMDEP_RW, /* CLKDM54XX_L4SEC */
	CLKDMDEP_RW, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_RW, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_mipiext_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_RW, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RW, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_RW, /* CLKDM54XX_L3INIT */
	CLKDMDEP_RW, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_RW, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_RW, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_dss_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RW, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_RW, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_cust_efuse_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_l3init_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_RW, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_RW, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RW, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_RW, /* CLKDM54XX_L4PER */
	CLKDMDEP_RW, /* CLKDM54XX_L4SEC */
	CLKDMDEP_RW, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_RW, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_l4per_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_l4sec_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RW, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_RW, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_abe_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_dsp_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_RW, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_RO, /* CLKDM54XX_COREAON */
	CLKDMDEP_RO, /* CLKDM54XX_CAM */
	CLKDMDEP_RW, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RW, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_RW, /* CLKDM54XX_L3INIT */
	CLKDMDEP_RW, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_RW, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_RW, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_gpu_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RW, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_RW, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_iva_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RW, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_mpu_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_RW, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_RO, /* CLKDM54XX_COREAON */
	CLKDMDEP_RO, /* CLKDM54XX_CAM */
	CLKDMDEP_RW, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RW, /* CLKDM54XX_EMIF */
	CLKDMDEP_RW, /* CLKDM54XX_IPU */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RW, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_RO, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_RO, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_RW, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_RW, /* CLKDM54XX_L3INIT */
	CLKDMDEP_RW, /* CLKDM54XX_L4PER */
	CLKDMDEP_RW, /* CLKDM54XX_L4SEC */
	CLKDMDEP_RW, /* CLKDM54XX_ABE */
	CLKDMDEP_RW, /* CLKDM54XX_DSP */
	CLKDMDEP_RW, /* CLKDM54XX_GPU */
	CLKDMDEP_RW, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_none_statdep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_emu_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_wkupaon_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_coreaon_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_cam_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_l4cfg_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_RO, /* CLKDM54XX_COREAON */
	CLKDMDEP_RO, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RO, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_RO, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_RO, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_RO, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_RO, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_RO, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_emif_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_ipu_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_RO, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_l3main2_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_RO, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_RO, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_RO, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_RO, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_RO, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_RO, /* CLKDM54XX_L4PER */
	CLKDMDEP_RO, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_RO, /* CLKDM54XX_GPU */
	CLKDMDEP_RO, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_instr_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_l3main1_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_RO, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_RO, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RO, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_RO, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_c2c_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RO, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_dma_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_mipiext_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RO, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_dss_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_cust_efuse_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_l3init_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_l4per_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_RO, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_RO, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_RO, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_l4sec_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_abe_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_dsp_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_RO, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_RO, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_gpu_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_iva_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_mpu_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_RO, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_RO, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_RO, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type clkdmdep54xx_none_dyndep_table[CLKDM54XX_ID_MAX] = {
	CLKDMDEP_NA, /* CLKDM54XX_EMU */
	CLKDMDEP_NA, /* CLKDM54XX_WKUPAON */
	CLKDMDEP_NA, /* CLKDM54XX_COREAON */
	CLKDMDEP_NA, /* CLKDM54XX_CAM */
	CLKDMDEP_NA, /* CLKDM54XX_L4CFG */
	CLKDMDEP_NA, /* CLKDM54XX_EMIF */
	CLKDMDEP_NA, /* CLKDM54XX_IPU */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN2 */
	CLKDMDEP_NA, /* CLKDM54XX_INSTR */
	CLKDMDEP_NA, /* CLKDM54XX_L3MAIN1 */
	CLKDMDEP_NA, /* CLKDM54XX_C2C */
	CLKDMDEP_NA, /* CLKDM54XX_DMA */
	CLKDMDEP_NA, /* CLKDM54XX_MIPIEXT */
	CLKDMDEP_NA, /* CLKDM54XX_DSS */
	CLKDMDEP_NA, /* CLKDM54XX_CUST_EFUSE */
	CLKDMDEP_NA, /* CLKDM54XX_L3INIT */
	CLKDMDEP_NA, /* CLKDM54XX_L4PER */
	CLKDMDEP_NA, /* CLKDM54XX_L4SEC */
	CLKDMDEP_NA, /* CLKDM54XX_ABE */
	CLKDMDEP_NA, /* CLKDM54XX_DSP */
	CLKDMDEP_NA, /* CLKDM54XX_GPU */
	CLKDMDEP_NA, /* CLKDM54XX_IVA */
	CLKDMDEP_NA, /* CLKDM54XX_MPU */
	CLKDMDEP_NA}; /* CLKDM54XX_NONE */


const clkdmdep_ctrl_type *clkdmdep54xx_statdep_table[CLKDM54XX_ID_MAX] = {
	clkdmdep54xx_emu_statdep_table, /* CLKDM54XX_EMU */
	clkdmdep54xx_wkupaon_statdep_table, /* CLKDM54XX_WKUPAON */
	clkdmdep54xx_coreaon_statdep_table, /* CLKDM54XX_COREAON */
	clkdmdep54xx_cam_statdep_table, /* CLKDM54XX_CAM */
	clkdmdep54xx_l4cfg_statdep_table, /* CLKDM54XX_L4CFG */
	clkdmdep54xx_emif_statdep_table, /* CLKDM54XX_EMIF */
	clkdmdep54xx_ipu_statdep_table, /* CLKDM54XX_IPU */
	clkdmdep54xx_l3main2_statdep_table, /* CLKDM54XX_L3MAIN2 */
	clkdmdep54xx_instr_statdep_table, /* CLKDM54XX_INSTR */
	clkdmdep54xx_l3main1_statdep_table, /* CLKDM54XX_L3MAIN1 */
	clkdmdep54xx_c2c_statdep_table, /* CLKDM54XX_C2C */
	clkdmdep54xx_dma_statdep_table, /* CLKDM54XX_DMA */
	clkdmdep54xx_mipiext_statdep_table, /* CLKDM54XX_MIPIEXT */
	clkdmdep54xx_dss_statdep_table, /* CLKDM54XX_DSS */
	clkdmdep54xx_cust_efuse_statdep_table, /* CLKDM54XX_CUST_EFUSE */
	clkdmdep54xx_l3init_statdep_table, /* CLKDM54XX_L3INIT */
	clkdmdep54xx_l4per_statdep_table, /* CLKDM54XX_L4PER */
	clkdmdep54xx_l4sec_statdep_table, /* CLKDM54XX_L4SEC */
	clkdmdep54xx_abe_statdep_table, /* CLKDM54XX_ABE */
	clkdmdep54xx_dsp_statdep_table, /* CLKDM54XX_DSP */
	clkdmdep54xx_gpu_statdep_table, /* CLKDM54XX_GPU */
	clkdmdep54xx_iva_statdep_table, /* CLKDM54XX_IVA */
	clkdmdep54xx_mpu_statdep_table, /* CLKDM54XX_MPU */
	clkdmdep54xx_none_statdep_table}; /* CLKDM54XX_NONE */

const clkdmdep_ctrl_type *clkdmdep54xx_dyndep_table[CLKDM54XX_ID_MAX] = {
	clkdmdep54xx_emu_dyndep_table, /* CLKDM54XX_EMU */
	clkdmdep54xx_wkupaon_dyndep_table, /* CLKDM54XX_WKUPAON */
	clkdmdep54xx_coreaon_dyndep_table, /* CLKDM54XX_COREAON */
	clkdmdep54xx_cam_dyndep_table, /* CLKDM54XX_CAM */
	clkdmdep54xx_l4cfg_dyndep_table, /* CLKDM54XX_L4CFG */
	clkdmdep54xx_emif_dyndep_table, /* CLKDM54XX_EMIF */
	clkdmdep54xx_ipu_dyndep_table, /* CLKDM54XX_IPU */
	clkdmdep54xx_l3main2_dyndep_table, /* CLKDM54XX_L3MAIN2 */
	clkdmdep54xx_instr_dyndep_table, /* CLKDM54XX_INSTR */
	clkdmdep54xx_l3main1_dyndep_table, /* CLKDM54XX_L3MAIN1 */
	clkdmdep54xx_c2c_dyndep_table, /* CLKDM54XX_C2C */
	clkdmdep54xx_dma_dyndep_table, /* CLKDM54XX_DMA */
	clkdmdep54xx_mipiext_dyndep_table, /* CLKDM54XX_MIPIEXT */
	clkdmdep54xx_dss_dyndep_table, /* CLKDM54XX_DSS */
	clkdmdep54xx_cust_efuse_dyndep_table, /* CLKDM54XX_CUST_EFUSE */
	clkdmdep54xx_l3init_dyndep_table, /* CLKDM54XX_L3INIT */
	clkdmdep54xx_l4per_dyndep_table, /* CLKDM54XX_L4PER */
	clkdmdep54xx_l4sec_dyndep_table, /* CLKDM54XX_L4SEC */
	clkdmdep54xx_abe_dyndep_table, /* CLKDM54XX_ABE */
	clkdmdep54xx_dsp_dyndep_table, /* CLKDM54XX_DSP */
	clkdmdep54xx_gpu_dyndep_table, /* CLKDM54XX_GPU */
	clkdmdep54xx_iva_dyndep_table, /* CLKDM54XX_IVA */
	clkdmdep54xx_mpu_dyndep_table, /* CLKDM54XX_MPU */
	clkdmdep54xx_none_dyndep_table}; /* CLKDM54XX_NONE */
